import dataclasses
import functools
import os
import sys
import typing as t

import attr
import click
import desert
import glom
import inflection
import lark
import marshmallow
import typing_inspect

import clout.exceptions

from .. import _util
from . import parsing


NO_DEFAULT = "__NO_DEFAULT__"
TOP_LEVEL_NAME = "top_level_name"
NoneType = type(None)
Dataclass = t.NewType("Dataclass", type)


def is_python_syntax(s: str) -> bool:
    import black

    try:
        black.format_str(s, 88)
    except black.InvalidInput:
        return False
    return True


def pythonify(obj):
    if is_python_syntax(repr(obj)):
        return repr(obj)
    if callable(obj) and is_python_syntax(obj.__name__):
        return obj.__name__
    return "..."


class Debug:
    def __repr__(self):
        import black

        pairs = ", ".join(
            (
                f"{k}={pythonify(v)}"
                for k, v in vars(self).items()
                if not k.startswith("_")
            )
        )
        name = type(self).__name__
        return black.format_str(f"{name}({pairs})", line_length=88)


class Option(Debug, click.Option):
    pass


class Group(Debug, parsing.CountingGroup):
    def __call__(self, *args, **kwargs):
        try:
            return super().__call__(*args, **kwargs, standalone_mode=False)
        except SystemExit as e:
            if e.code != 0:
                raise


class DebuggableCommand(Debug, parsing.CountingCommand):
    def __call__(self, *args, **kwargs):
        try:
            return super().__call__(*args, **kwargs)
        except SystemExit as e:
            if e.code != 0:
                raise


class MarshmallowFieldParam(click.ParamType):
    def __init__(self, field):
        self.field = field

    @property
    def name(self):
        return type(self.field).__name__.split(".")[-1]

    def convert(self, value, param, ctx):

        try:
            return self.field.deserialize(value)
        except marshmallow.exceptions.ValidationError as e:
            raise click.BadParameter(ctx=ctx, param=param, message=str(e)) from e


MM_TO_CLICK = {
    marshmallow.fields.String: click.STRING,
    marshmallow.fields.Int: click.INT,
    marshmallow.fields.Float: click.FLOAT,
    marshmallow.fields.Raw: click.STRING,
    marshmallow.fields.Bool: click.BOOL,
}


def make_help_command():
    return DebuggableCommand(name="--help", hidden=True)


@functools.singledispatch
def make_param_from_field(
    field: marshmallow.fields.Field, data, default
) -> click.Parameter:

    if data:

        data = data.copy()
        if "type" not in data:
            data["type"] = MM_TO_CLICK[type(field)]

        return Option(show_default=True, **data)

    return Option(
        ["--" + field.name],
        type=MarshmallowFieldParam(field),
        required=False,
        default=default,
        show_default=True,
    )


@make_param_from_field.register(marshmallow.fields.Boolean)
def _(field: marshmallow.fields.Boolean, data, default) -> Option:

    if data:
        return Option(**data, default=default)

    field_name = _util.dasherize(field.name)
    required = field.missing == marshmallow.missing == field.default == default
    return Option(
        [f"--{field_name}/--no-{field_name}"],
        default=default,
        required=required,
        is_flag=True,
        show_default=not required,
    )


@make_param_from_field.register(marshmallow.fields.String)
@make_param_from_field.register(marshmallow.fields.Int)
@make_param_from_field.register(marshmallow.fields.Float)
@make_param_from_field.register(marshmallow.fields.Date)
@make_param_from_field.register(marshmallow.fields.DateTime)
@make_param_from_field.register(marshmallow.fields.Raw)
def _(field, data, default) -> Option:
    if data:
        data = data.copy()
        if "type" not in data:
            data["type"] = MM_TO_CLICK[type(field)]
        return Option(show_default=True, **data, default=default)
    param_type = MM_TO_CLICK[type(field)]
    return Option(
        ["--" + field.name], type=param_type, show_default=True, default=default
    )


def extract(mapping, path):
    for entry in path:
        mapping = mapping[entry]
    return mapping


def get_default(field, path, default_map):
    # XXX The envvar logic should be somewhere else.
    envvar = field.metadata.get("clout", {}).get("cli", {}).get("envvar")
    if envvar is not None:
        value = os.environ.get(envvar)
        if value is not None:
            return value

    try:
        value = extract(default_map, path[1:])
        return value
    except KeyError:
        return field.default


@attr.dataclass(frozen=True)
class CLI:
    context_settings: t.Dict[str, t.Any] = attr.ib(factory=dict)
    inherits: t.FrozenSet[str] = frozenset({"app_name"})
    metadata_key: str = "cli"
    args: t.List[str] = attr.ib(factory=list)
    app_name: t.Optional[str] = None

    def make_command_from_schema(
        self, schema: marshmallow.Schema, path: t.Sequence[str]
    ) -> click.BaseCommand:
        params = []
        commands = []

        for field in schema.fields.values():

            if isinstance(field, marshmallow.fields.Nested):
                commands.append(
                    self.make_command_from_schema(
                        field.schema, path=path + (field.name,)
                    )
                )
            elif isinstance(field, marshmallow.fields.Field):
                user_specified = field.metadata.get("cli")
                default = get_default(
                    field,
                    path=path + (field.name,),
                    default_map=self.context_settings.get("default_map", {}),
                )

                if isinstance(field.metadata.get("cli"), click.Parameter):
                    param = field.metadata["cli"]

                else:

                    param = make_param_from_field(
                        field, user_specified, default=default
                    )

                params.append(param)
            else:
                raise TypeError(field)

        commands.append(make_help_command())

        help = getattr(schema, "help", None)
        if commands:
            return Group(
                name=path[-1],
                commands={c.name: c for c in commands},
                params=params,
                chain=True,
                result_callback=lambda *a, **kw: (a, kw),
                help=help,
                short_help=help,
                context_settings=self.context_settings,
            )
        return DebuggableCommand(
            name=path[-1],
            params=params,
            callback=identity,
            help=help,
            short_help=help,
            context_settings=self.context_settings,
        )

    def get_command(
        self,
        typ: t.Type,
        default=NO_DEFAULT,
        metadata: t.Mapping[str, t.Any] = None,
        args=(),
    ):
        metadata = metadata or {}

        cli_metadata: t.Union[
            t.Dict[str, t.Any], click.BaseCommand, click.Parameter
        ] = metadata.get(self.metadata_key, None)
        if isinstance(cli_metadata, (click.BaseCommand, click.Parameter)):
            command = cli_metadata
        else:

            name = metadata.get("name", _util.dasherize(self.app_name))

            schema = desert.schema_class(typ)()
            command = self.make_command_from_schema(schema, path=(name,))

            def schema_load(*a, **kw):
                try:
                    return schema.load(*a, **kw)
                except marshmallow.exceptions.ValidationError as e:
                    raise clout.exceptions.ValidationError(*e.args) from e

            command.callback = schema_load

        command = Group(
            name=TOP_LEVEL_NAME,
            commands={c.name: c for c in [command, make_help_command()]},
            callback=command.callback,
        )
        return command

    def prep(
        self,
        typ: t.Type,
        default=NO_DEFAULT,
        metadata: t.Mapping[str, t.Any] = None,
        args=(),
    ):

        command = self.get_command(typ, default, metadata, args)

        parser = parsing.Parser(command, callback=command.callback, use_defaults=True)
        cli_args = (TOP_LEVEL_NAME, _util.dasherize(typ.__name__)) + tuple(
            args or self.args or sys.argv[2:]
        )

        try:
            result = parser.parse_args(cli_args)
        except click.exceptions.BadParameter as e:
            e.show()
            sys.exit(1)
        except (
            lark.exceptions.ParseError,
            lark.exceptions.UnexpectedCharacters,
            lark.exceptions.VisitError,
        ):

            print(command.get_help(click.Context(command)) + EPILOG)
            if int(os.environ.get("CLI_SHOW_TRACEBACK", 0)):
                raise
            else:
                sys.exit(1)
        except clout.exceptions.MissingInput as e:
            result = parser.parse_args(e.found + ["--help"])

        [value] = result.values()

        return value

    def build(
        self,
        typ: t.Type,
        default=NO_DEFAULT,
        metadata: t.Mapping[str, t.Any] = None,
        args=(),
    ):

        command = self.get_command(typ, default, metadata, args)

        result = command.callback(self.prep(typ, default, metadata, args))

        return result

    def set(self, **kw):
        return attr.evolve(self, **kw)


class NonStandaloneCommand(click.Command):
    def main(self, *a, standalone_mode=False, **kw):
        return super().main(*a, standalone_mode=standalone_mode, **kw)


EPILOG = "\n\nNote:\n  export CLI_SHOW_TRACEBACK=1 to show traceback on error.\n"


class Command(click.Command):
    """A :class:`click.Command` built from an :func:`attr.dataclass` or :func:`dataclasses.dataclass`."""

    def __init__(
        self,
        type,
        *args,
        name=None,
        app_name=None,
        callback=lambda x: x,
        params=None,
        context_settings=None,
        epilog=None,
        **kwargs,
    ):
        if not (attr.has(type) or dataclasses.is_dataclass(type)):
            raise TypeError(f"Need a dataclass, got {type} of type {type.__class__}")
        self.app_name = app_name

        epilog = epilog or ""
        epilog += EPILOG

        context_settings = context_settings or {}
        context_settings["ignore_unknown_options"] = True
        super().__init__(
            name=name,
            *args,
            **kwargs,
            add_help_option=False,
            epilog=epilog,
            context_settings=context_settings,
        )
        self.params = (params or []) + [
            click.Argument(["args"], type=click.UNPROCESSED, nargs=-1)
        ]
        self.callback = lambda args: callback(
            CLI(
                app_name=app_name or _util.dasherize(type.__name__),
                context_settings=context_settings,
            ).build(type, args=args)
        )

    def build(self):
        """Return an instance of `self.type`, built from the command line arguments."""
        return self.main(standalone_mode=False)

    def main(self, **kwargs):
        """Run the command and exit the program afterwards.

        Upcalls directly to :meth:`click.MultiCommand.main()`.
        """
        return super().main(**kwargs)


def command(type: Dataclass, **kwargs):
    """A decorator that replaces the decorated function with a :class:`clout.Command`.

    The with `callback` attribute is set to the decorated function. Compare to
    :func:`click.command()`.

    """

    def decorator(callback):
        return Command(type, callback=callback, **kwargs)

    return decorator



__version__ = "0.1.13"

__all__ = ["Command", "load_env", "DeepChainMap"]

from ._loaders.cli import Command
from ._loaders.cli import command
from ._loaders.env import load_env
from ._loaders.multi import DeepChainMap



import os
import typing as t

import attr
import desert
import glom
import inflection
import marshmallow

from .. import _util


@attr.dataclass
class Env:
    app_name: str = None
    metadata_key: str = "env"
    inherits: t.List[str] = frozenset({"app_name"})
    env: t.Dict[str, t.Any] = attr.ib(factory=dict)
    prefix: str = ""

    def make_path_to_field(
        self, schema: marshmallow.Schema, path=()
    ) -> t.Dict[str, marshmallow.fields.Field]:

        d = {}
        for field in schema.fields.values():

            if isinstance(field, marshmallow.fields.Nested):
                recursed = self.make_path_to_field(
                    field.schema, path=path + (field.name,)
                )
                d.update(recursed)

            elif isinstance(field, marshmallow.fields.Field):
                d[path + (field.name,)] = field
            else:
                raise TypeError(field)

        return d

    def make_envvar_name(self, path: t.Tuple[str]) -> str:
        prefix = self.prefix if self.prefix else self.app_name
        return inflection.underscore("_".join((prefix,) + path)).upper()

    def prep(self, typ, metadata=None, default=None, env=None, name=None):
        # TODO make sure this handles lists correctly.
        # If a field is a list, this should return a list, not a single member.
        metadata = metadata or {}

        schema = desert.schema_class(typ)()

        path_to_field = self.make_path_to_field(schema, path=())

        d = {}
        for path, field in path_to_field.items():
            name = self.make_envvar_name(path)

            value = os.environ.get(name)

            if value is not None:
                d[path] = field.deserialize(value)

        nested = make_nested(d)

        try:
            return nested
        except KeyError:
            return {}

    def set(self, **kw):
        return attr.evolve(self, **kw)


def make_nested(path_to_value: t.Dict[t.Tuple, t.Any]) -> dict:
    d = {}
    for path, value in sorted(
        path_to_value.items(), key=lambda path_value: len(path_value[0])
    ):
        func = dict
        glom.assign(d, ".".join(path), value, missing=func)
    return d


def load_env(type: t.Type, prefix: str = ""):
    """Load environment variables for a class into a dict.

    For example, define a class


    .. code-block:: python

        import attr


        @attr.dataclass
        class Database:
            host: str
            port: int


    Load the environment variables into a dict, setting the prefix for our app.

    .. code-block:: python

            d = clout.load_env(Database, prefix='MYAPP')
            assert d == {'host': 'example.com', 'port': 1234}


    Run the app with environment variables set.

    .. code-block:: bash

        export MYAPP_HOST=example.com
        export MYAPP_PORT=1234

    """
    return Env(prefix=prefix).prep(type)



import collections
import functools
import math
import os
import shlex
import subprocess
import sys
import typing as t

import attr
import click
import lark

import clout.exceptions

from .. import _util


ALWAYS_ACCEPT = True

HELP_COMMAND = "HELP_COMMAND"


class CountingBaseCommand:
    def __init__(self, *args, nargs=1, required=False, **kwargs):
        self.nargs = nargs
        self.required = required

        super().__init__(*args, **kwargs)

    @property
    def multiple(self):
        return self.nargs != 1


class CountingGroup(CountingBaseCommand, click.Group):
    pass


class CountingCommand(CountingBaseCommand, click.Command):
    pass


@functools.singledispatch
def to_lark(obj: object):
    raise NotImplementedError(obj)


def quote(s):
    return '"' + s + '"'


def par(s):
    return f"({s})"


def one_of(items):
    items = list(items)
    if len(items) == 1:
        return str(items[0])
    return par(" | ".join(items))


def name_rule(obj) -> str:
    return f"{obj.name.lstrip('-').replace('-', '_')}_{id(obj)}"


@to_lark.register
def _(option: click.Option):
    if option.is_flag:
        return (
            f"{name_rule(option)} : "
            + "|".join(quote(decl) for decl in option.opts)
            + "\n"
        )
    return (
        f"{name_rule(option)} : "
        + one_of(quote(decl) + ' "="? value' for decl in option.opts)
        + "\n"
    )


@to_lark.register
def _(arg: click.Argument):
    return f"{name_rule(arg)} : value\n"


def min_params(cmd: click.BaseCommand) -> int:
    return sum(
        1 if p.nargs == -1 or p.multiple else p.nargs
        for p in cmd.params
        if p.required or not p.default
    )


def max_params(cmd: click.BaseCommand) -> t.Union[int, float]:
    return sum(math.inf if p.nargs == -1 or p.multiple else p.nargs for p in cmd.params)


@to_lark.register
def _(cmd: CountingCommand):
    optionals = [name_rule(p) for p in cmd.params if not p.required or p.default]
    requireds = [name_rule(p) for p in cmd.params if p.required]
    params = one_of(optionals + requireds)

    if max_params(cmd) == math.inf:
        params += f"+"
    else:
        params += f" ~ {min_params(cmd)}..{max_params(cmd)}"
    out = f"{name_rule(cmd)} : {quote(cmd.name)} {params} \n"
    for p in cmd.params:
        out += to_lark(p)
    return out


@to_lark.register
def _(grp: CountingGroup):
    if not grp.commands:
        return to_lark.dispatch(CountingCommand)(grp)
    command = one_of(name_rule(c) for c in grp.commands.values())

    optionals = [name_rule(p) for p in grp.params if not p.required]
    requireds = [name_rule(p) for p in grp.params if p.required]
    params = one_of(optionals + requireds)

    if grp.params:
        out = f"{name_rule(grp)} : {quote(grp.name)} ({command}|{params})+ \n"
    else:
        out = f"{name_rule(grp)} : {quote(grp.name)} {command}+ \n"

    for c in grp.commands.values():
        out += to_lark(c)
    for p in grp.params:
        out += to_lark(p)
    return out


def build_grammar(grp):
    grammar = to_lark(grp)
    grammar += "?value : /\S+/\n"
    grammar += f"?start : {name_rule(grp)}\n"
    grammar += "%import common.CNAME\n"
    grammar += "%import common.WS -> _WHITE\n"
    grammar += "%ignore _WHITE\n"

    return grammar


def get_base_commands(grp: click.BaseCommand) -> t.Iterator[click.BaseCommand]:
    yield grp
    if not hasattr(grp, "commands"):
        return
    for cmd in grp.commands.values():
        yield from get_base_commands(cmd)


class Walker(lark.Visitor):
    def __init__(self, *args, group, **kwargs):
        self.group = group
        super().__init__(*args, **kwargs)
        base_commands = list(get_base_commands(self.group))
        self.all_param_names = {name_rule(p) for c in base_commands for p in c.params}
        for cmd in base_commands:
            name, method = self.make_validation_method(cmd)
            setattr(self, name, method)

    def make_validation_method(self, command):
        def param_validation_method(parsed_command):

            counter = collections.Counter(p.data for p in parsed_command.children)

            for param_or_cmd in list(command.params) + list(
                getattr(command, "commands", {}).values()
            ):
                param_or_cmd_id = name_rule(param_or_cmd)
                observed = counter.get(param_or_cmd_id, 0)
                if observed > param_or_cmd.nargs:
                    raise TooManyArgs(param_or_cmd)

        return name_rule(command), param_validation_method


class CLIParsingErrorException(Exception):
    pass


class InvalidInput(CLIParsingErrorException):
    pass


class HelpRequested(Exception):
    pass


class TooManyArgs(CLIParsingErrorException):
    pass


class AmbiguousArgs(CLIParsingErrorException):
    pass


class Transformer(lark.Transformer):
    def __init__(self, *args, group, use_defaults, **kwargs):
        self.group = group
        self.use_defaults = use_defaults
        super().__init__(*args, **kwargs)
        base_commands = list(get_base_commands(self.group))
        self.all_param_names = {name_rule(p) for c in base_commands for p in c.params}
        for cmd in base_commands:
            if isinstance(cmd, CountingCommand) and not isinstance(
                cmd, click.MultiCommand
            ):
                # Not a group
                name, method = self.make_command_method(cmd)
                setattr(self, name, method)
            for param in cmd.params:
                name, method = self.make_param_method(cmd, param)
                setattr(self, name, method)

            if isinstance(cmd, click.MultiCommand):
                name, method = self.make_group_method(cmd)
                setattr(self, name, method)

    def process_params(self, command, parsed):

        d = {}
        for param, value in parsed:

            if param.name == "--help":
                print(command.get_help(click.Context(command)))
                sys.exit()
            if isinstance(param, click.Parameter):
                value = str(value)
            if param.multiple or param.nargs != 1:
                d.setdefault(param, []).append(value)
            else:
                d[param] = value

        out = {}
        for param, value in d.items():
            if isinstance(param, click.Parameter):
                try:
                    out[param.name] = param.process_value(click.Context(command), value)
                except click.exceptions.BadParameter as e:
                    # XXX
                    raise
            elif isinstance(param, click.BaseCommand):

                out[param.name] = {k: v for k, v in value.items() if v != _util.UNSET}
            else:
                raise TypeError(param)

        if self.use_defaults:
            for param in command.params:
                if param.name not in out and not param.required:

                    out[param.name] = param.default

        return command, out

    def make_command_method(self, command):

        return (name_rule(command), lambda parsed: self.process_params(command, parsed))

    def make_group_method(self, group):
        def method(parsed):
            import lark

            _group, out = self.process_params(
                group, [(obj, value) for obj, value in parsed]
            )
            return (group, out)

        return name_rule(group), method

    def make_param_method(self, cmd, param):
        def method(parsed):
            if param.is_flag:
                parsed = True
            else:
                [parsed] = parsed

            return (param, parsed)

        return name_rule(param), method


class RemoveInvalidBranches(lark.Transformer):
    def __init__(self, *args, group, **kwargs):
        self.group = group
        super().__init__(*args, **kwargs)
        base_commands = list(get_base_commands(self.group))
        self.all_param_names = {name_rule(p) for c in base_commands for p in c.params}

    def _ambig(self, data):

        trees = [tree for tree in data if check_validity(self.group, tree)]
        if len(trees) == 1:
            return trees[0]

        if len(trees) == 0:
            raise InvalidInput(data)

        trees = [tree for tree in trees if has_help(tree)]

        if not trees:
            raise AmbiguousArgs(data)

        # Pick one.
        return trees[-1]


def has_help(tree):
    return bool(tree.find_data(HELP_COMMAND))


def check_validity(group, tree):
    try:
        Walker(group=group).visit(tree)
    except TooManyArgs:
        return False
    return True


def find_missing_input(parser, s: str) -> t.Optional[t.List[str]]:

    words = shlex.split(s)
    try:
        parser.parse(subprocess.list2cmdline(words))
    except lark.exceptions.ParseError:
        pass
    else:
        return None

    while True:
        try:
            parser.parse(subprocess.list2cmdline(words[:-1]))
        except lark.exceptions.ParseError:
            words = words[:-1]
        else:
            return words


@attr.dataclass
class Parser:
    group: CountingGroup
    callback: t.Callable = lambda **kw: kw
    use_defaults: bool = True
    _id_to_object: t.Dict[str, object] = attr.ib(factory=dict)

    def __attrs_post_init__(self):
        for cmd in get_base_commands(self.group):
            self._id_to_object[name_rule(cmd)] = cmd
            for param in cmd.params:
                self._id_to_object[name_rule(param)] = param

    def parse_string(self, s):
        grammar = build_grammar(self.group)
        if int(os.environ.get("CLI_SHOW_GRAMMAR", 0)):
            print(grammar)
        parser = lark.Lark(grammar, parser="earley", ambiguity="explicit")
        try:
            tree = parser.parse(s)
        except lark.exceptions.ParseError as e:
            found = find_missing_input(parser, s)
            raise clout.exceptions.MissingInput(self.group, s, found) from e

        try:
            tree = RemoveInvalidBranches(group=self.group).transform(tree)
        except AmbiguousArgs:
            click.echo(
                "The command arguments were ambiguous. Rearranging terms might help."
            )

        transformer = Transformer(group=self.group, use_defaults=self.use_defaults)

        try:
            _group, value = transformer.transform(tree)
        except lark.exceptions.VisitError as e:
            raise e.orig_exc from e

        return value

    def parse_args(self, args: t.List[str]):
        line = subprocess.list2cmdline(args)
        return self.parse_string(line)



import click
import click._compat


class CloutException(Exception):
    """Base exception for Clout exceptions."""


class ValidationError(CloutException):
    """Raised for invalid command-line arguments."""


class MissingInput(CloutException):
    """Raised for missing command-line arguments."""

    def __init__(self, group, string, found):
        self.group = group
        self.string = string
        self.found = found

    def __str__(self):
        return f"Parsing fails at {self.found}"



import collections
import collections.abc
import typing as t

import attr
import desert
import marshmallow

from .. import exceptions


@attr.dataclass(frozen=True)
class Multi:
    loaders: t.List = attr.ib(factory=list)
    inherits: t.FrozenSet[str] = attr.ib(default=frozenset())
    metadata_key: str = "cli"
    data: dict = attr.ib(factory=dict)

    def set(self, **kw):
        return attr.evolve(self, **kw)

    def set_data_on_loaders(self,):
        loaders = []
        for loader in self.loaders:
            loaders.append(
                loader.set(
                    **{k: v for k, v in self.data.items() if k in loader.inherits}
                )
            )
        return self.set(loaders=loaders)

    def prep(self, cls):
        multi = self.set_data_on_loaders()
        return DeepChainMap(*[loader.prep(cls) or {} for loader in multi.loaders])

    def build(self, cls):
        schema = desert.schema_class(cls)()
        prepped = self.prep(cls)
        try:
            return schema.load(prepped)
        except marshmallow.exceptions.ValidationError as e:
            raise exceptions.ValidationError(*e.args) from e


class DeepChainMap(collections.ChainMap):
    """Combine multiple dicts into a deep mapping.

    Lookups that fail in the first dict will be checked in the next one.


    >>> import clout
    >>> maps = [{"a": {}}, {"a": {"b": {}}}, {"a": {"b": {"c": 1337}}}]
    >>> dcm = clout.DeepChainMap(*maps)
    >>> dcm["a"]["b"]["c"]
    1337


    """

    def __getitem__(self, key):
        value = super().__getitem__(key)
        if isinstance(value, collections.abc.Mapping):
            return type(self)(*[m[key] for m in self.maps if key in m])
        return value



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['clout', 'clout._loaders']

package_data = \
{'': ['*']}

install_requires = \
['appdirs>=1.4,<2.0',
 'attrs>=19.2.0',
 'click',
 'dataclasses>=0.6.0,<0.7.0',
 'desert>=2019.11.06',
 'glom>=19.2,<20.0',
 'importlib_resources>=1.0,<2.0',
 'inflection>=0.3.1,<0.4.0',
 'lark-parser>=0.7.3,<0.8.0',
 'marshmallow>=3.0,<4.0',
 'typing-extensions>=3.7,<4.0',
 'typing-inspect>=0.4.0,<0.5.0']

setup_kwargs = {
    'name': 'clout',
    'version': '0.1.13',
    'description': 'Command-line Object Utility Tool',
    'long_description': "=======================================================\nClout: Command-Line Object Utility Tool\n=======================================================\n\n.. start-badges\n\n.. list-table::\n    :stub-columns: 1\n\n\n    * - docs\n      - |docs|\n    * - code\n      - |github|\n    * - tests\n      - | |travis|\n        | |codecov|\n    * - package\n      - | |version|\n        | |wheel|\n        | |supported-versions|\n        | |supported-implementations|\n        | |commits-since|\n\n.. |docs| image:: https://img.shields.io/readthedocs/clout\n    :target: https://clout.readthedocs.org\n    :alt: Documentation Status\n\n\n.. |travis| image:: https://img.shields.io/travis/com/python-clout/clout\n    :alt: Travis-CI Build Status\n    :target: https://travis-ci.com/python-clout/clout\n\n.. |version| image:: https://img.shields.io/pypi/v/clout.svg\n    :alt: PyPI Package latest release\n    :target: https://pypi.org/pypi/clout\n\n.. |commits-since| image:: https://img.shields.io/github/commits-since/python-clout/clout/v0.1.13.svg\n    :alt: Commits since latest release\n    :target: https://github.com/python-clout/clout/compare/v0.1.13...master\n\n.. |wheel| image:: https://img.shields.io/pypi/wheel/clout.svg\n    :alt: PyPI Wheel\n    :target: https://pypi.org/pypi/clout\n\n.. |supported-versions| image:: https://img.shields.io/pypi/pyversions/clout.svg\n    :alt: Supported versions\n    :target: https://pypi.org/pypi/clout\n\n.. |supported-implementations| image:: https://img.shields.io/pypi/implementation/clout.svg\n    :alt: Supported implementations\n    :target: https://pypi.org/pypi/clout\n\n.. |codecov| image:: https://img.shields.io/codecov/c/github/python-clout/clout/master.svg\n     :alt: Coverage\n     :target: https://codecov.io/gh/python-clout/clout\n\n.. |github| image:: https://img.shields.io/github/last-commit/python-clout/clout\n     :alt: Last commit\n     :target: https://img.shields.io/github/last-commit/python-clout/clout\n\n.. end-badges\n\n\n\n..\n    start-usage\n\n\nConvert dataclasses into a command-line interface.\n\nQuickstart\n---------------\n\n\nTo install, use\n\n.. code-block:: bash\n\n    pip install clout\n\n\nDefine some dataclasses and convert them into a command-line interface.\n\n\n.. code-block:: python\n\n    import attr\n    import click\n\n    import clout\n\n\n    @attr.dataclass\n    class DB:\n        host: str\n        port: int\n\n\n    @attr.dataclass\n    class Config:\n        db: DB\n        dry_run: bool\n\n\n    cli = clout.Command(Config)\n\n    print(cli.build())\n\n\n.. code-block:: bash\n\n    $ myapp --dry-run db --host example.com --port 9999\n    Config(db=DB(host='example.com', port=9999), dry_run=True)\n\n\n..\n    end-usage\n\nSee the full docs for more information: https://clout.readthedocs.io/\n",
    'author': 'Clout contributors',
    'author_email': 'python-clout@users.noreply.github.com',
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
